// DIGAHttpCommunicator.java
// DIGA HTTP Communicator 1.1
// 2013.3.16 (SAT)
// 北の電話男 Kita no Denwa-otoko
// http://www.geocities.jp/kitano_denwaotoko/

import java.net.*;
import java.io.*;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

class DIGAHttpException extends Exception {
	protected int errorCode;
	public DIGAHttpException(String str) {
		super(str);
		errorCode = 0;
	}
	
	public DIGAHttpException(String str, int err) {
		this("DIGA-HTTP例外" + err + " : " + str);
		errorCode = err;
	}
	
	public int getErrorCode() {
		return errorCode;
	}
}

class DIGAHttpDataFormatException extends DIGAHttpException {
	public DIGAHttpDataFormatException (String str) { super(str); }
	public DIGAHttpDataFormatException (String str, int err) {
		this("DIGA-HTTP データ不正" + err + " : " + str);
		errorCode = err;
	}
}

class DIGAHttpCommunicator {
	String host;
	int port;
	String userAgent;
	String password;
	String digest;
	int error;
	PrintStream pout; //デバッグ用全データ出力
	DIGAHttpLogDisplay lout; //経過表示用ログ出力
	String pLogout, pDTitle;
	
	public DIGAHttpCommunicator(String hostName, int portNumber, String password) {
		host = hostName;
		port = portNumber;
		userAgent = null;
		this.password = password;
		digest = null;
		error = 0;
		pout = null;
		lout = null;
		pLogout = pDTitle = null;
	}
	
	public void setPrintStream(PrintStream p) {
		pout = p;
	}
	
	public void setLogDisplay(DIGAHttpLogDisplay l) {
		lout = l;
	}
	
	public void setUserAgent(String userAgent) {
		this.userAgent = userAgent;
	}
	
	private String send(String message) throws DIGAHttpException {
		String res = null;
		StringBuffer buf = new StringBuffer(1000);
		try {
			Socket server = new Socket(host,port);
			InputStreamReader reader = new InputStreamReader(server.getInputStream(), "Shift_JIS");
			BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(server.getOutputStream(), "Shift_JIS"));
			int c;
			
			writer.write(message);
			writer.flush();
			if (pout != null) pout.printf("\n=== メッセージを送信します ===\n%s\n============\n\n",message);
			
			while ((c = reader.read()) != -1) {
				buf.append((char)c);
				//System.out.print((char)c);
			}
			
			res = buf.toString();
			error = 0;
			writer.close();
			reader.close();
			server.close();
		} catch (java.net.UnknownHostException e) {
			// res はもともとnull
			error = 2;
			throw new DIGAHttpException("指定されたホスト " + host + " が存在しません。",2);
		} catch (java.io.IOException e) {
			error = 1;
			if (buf.length() != 0) res = buf.toString();
			else throw new DIGAHttpException("入出力エラーによりDIGAとのデータ送受信に失敗しました。",2);
		}
		if (pout != null) pout.printf("\n%s\n", res);
		return res;
	}
	
	public int getError() {
		return error;
	}
	
	public String get(String path) throws DIGAHttpException {
		return send(
			"GET " + path + " HTTP/1.1\r\n" +
			((userAgent != null)?("User-Agent: " + userAgent + "\r\n"):"") +
			"Host: " + host + ":" + port + "\r\n\r\n"
		);
	}
	
	public String post(String path, String postData) throws DIGAHttpException {
		return send(
			"POST " + path + " HTTP/1.1\r\n" +
			((userAgent != null)?("User-Agent: " + userAgent + "\r\n"):"") +
			"Host: " + host + ":" + port + "\r\n" +
			"Content-Length: " + postData.length() + "\r\n\r\n" + 
			postData + "\r\n"
		);
	}
	
	public boolean login() throws DIGAHttpException, DIGAHttpDataFormatException {
		if (digest != null) { //既に認証済み
			error = 15;
			return false;
		}
		
		String loginPath = "/cgi-bin/prevLogin.cgi";
		String data;
		String nonce;
		
		if (lout != null) lout.appendLog("DIGAにログインします。認証用ナンス取得開始");
		if ((data = get(loginPath)) == null ||error != 0) { 
			error += 10; // send中のエラーに10足す
			throw new DIGAHttpException("DIGAとのデータ送受信に失敗しました。",error);
		}
		
		Pattern pnonce = Pattern.compile("var nonce=\"([0-9]+)\";");
		Matcher mnonce = pnonce.matcher(data);
		if (! mnonce.find()) { //認証用ナンスがない
			error = 16;
			throw new DIGAHttpDataFormatException("DIGAからの認証用データが見つかりません。",error);
		}
		nonce = mnonce.group(1);
		pnonce = null;
		mnonce = null;
		if (lout != null) lout.appendLog("認証用ナンス取得成功");
		
		if ((digest = Digest.bytesToB64Str(Digest.makeMD5Bytes(nonce + password))) == null) { //Digestがエラーを返した
			error = 17;
			throw new DIGAHttpException("内部データ処理エラーです。",error);
		}
		digest = "passwd=&nonce=" + nonce + "&digest=" +digest;
		
		if (lout != null) lout.appendLog("パスワード認証開始");
		if ((data = post("/cgi-bin/loginPsWd.cgi", digest)) == null || error != 0) { 
			error += 20; // send中のエラーに20足す
			throw new DIGAHttpException("パスワード認証中、DIGAとのデータ送受信に失敗しました",error);
		}
		if (data.contains("X-MEI-CONFIRM-DEVICE: NG") || data.contains("パスワードが間違っています。")) { //認証失敗
			error = 25;
			//throw new DIGAHttpException("DIGAのパスワード認証に失敗しました。",error);
			return false;
		} else if (data.contains("アクセスが禁止されています。")) {
			error = 26;
			return false;
		}
		if (lout != null) lout.appendLog("認証されました");
		
		if (lout != null) lout.appendLog("メニュー取得開始");
		getPath();
		if (lout != null) lout.appendLog("メニュー取得成功");
		
		error = 0;
		return true;
	}
	
	public boolean logout() throws DIGAHttpException { //前バージョンとの互換性維持のためboolean型。例外発生しなければ常にtrue
		digest = null;
		if (lout != null) lout.appendLog("DIGAからログアウトします");
		if (get(pLogout) == null  || error != 0) {
			error += 30;
			throw new DIGAHttpException("ログアウト処理中、DIGAとのデータ送受信に失敗しました",error);
		}
		error = 0;
		pLogout = pDTitle = null; //ゴミ収集へ
		if (lout != null) lout.appendLog("DIGAからログアウトしました");
		return true;
	}
	
	private void getPath() throws DIGAHttpException,DIGAHttpDataFormatException {
		if (digest == null) { //ログインしていない
			error = 40;
			throw new DIGAHttpException("ログインしていません。",error);
		}
		
		String data, path;
		if ((data = post("/cgi-bin/topMenu.cgi",digest)) == null || error != 0) {
			error += 40;
			throw new DIGAHttpException("操作メニューの受信中、DIGAとのデータ送受信に失敗しました。",error);
		}
		
		Pattern p = Pattern.compile("<frame name=\"leftframe\" src=\"(.+?)\"");
		Matcher m = p.matcher(data);
		if (!m.find()) { //メニューがない!
			error = 45;
			throw new DIGAHttpDataFormatException("DIGAからの操作メニューが不正です。",error);
		}
		path = m.group(1);
		
		if ((data = get(path)) == null || error != 0) {
			error += 50;
			throw new DIGAHttpException("操作メニューの受信中、DIGAとのデータ送受信に失敗しました。",error);
		}
		
		String folderPath, folderPath2, rdoprm, pnDTitle, pnLogout;
		/*
		JavaScript変数：
			var folderPath = "/cgi-bin/dispframe.cgi?DISP_PAGE=";
			var folderPath2 = "/cgi-bin/dispframe2.cgi?DISP_PAGE=";
			var rdoprm = "&Radio_Drive=1";
			var rdoval = "&Radio_Drive=";
		*/
		p = Pattern.compile("var folderPath = \"(.+?)\";");
		m = p.matcher(data);
		if (!m.find()) {
			error = 55;
			throw new DIGAHttpDataFormatException("DIGAからの操作メニューが不正です。",error);
		}
		folderPath = m.group(1);
		
		p = Pattern.compile("var folderPath2 = \"(.+?)\";");
		m = p.matcher(data);
		if (!m.find()) {
			error = 56;
			throw new DIGAHttpDataFormatException("DIGAからの操作メニューが不正です。",error);
		}
		folderPath2 = m.group(1);
		
		p = Pattern.compile("var rdoprm = \"(.+?)\";");
		m = p.matcher(data);
		if (!m.find()) {
			error = 57;
			throw new DIGAHttpDataFormatException("DIGAからの操作メニューが不正です。",error);
		}
		rdoprm = m.group(1);
		
		p = Pattern.compile("<input(.*?)name=\"Image_DTitle\" src=\"(.+?)\" onClick=\"JavaScript\\:clickDispPage\\(([0-9]+?)\\)\"");
		m = p.matcher(data);
		if (!m.find()) {
			error = 58;
			throw new DIGAHttpDataFormatException("DIGAからの操作メニューが不正です。",error);
		}
		pnDTitle = m.group(3);
		
		p = Pattern.compile("<input(.*?)name=\"Image_Logout\" src=\"(.*?)\" onClick=\"JavaScript:clickDispPage\\(([0-9]+?)\\)\"");
		m = p.matcher(data);
		if (!m.find()) {
			error = 59;
			throw new DIGAHttpDataFormatException("DIGAからの操作メニューが不正です。",error);
		}
		pnLogout = m.group(3);
		
		pDTitle = folderPath2 + pnDTitle + rdoprm;
		pLogout = folderPath + pnLogout + rdoprm;
		error = 0;
	}
	
	public String getDTitleCSV() throws DIGAHttpException,DIGAHttpDataFormatException {
		String data, path, pdata = null;
		StringBuffer buf;
		StringBuffer result = new StringBuffer(1000);
		
		if (digest == null || pDTitle == null) {
			error = 60;
			throw new DIGAHttpException("ログインしていません。",error);
		}
		
		if ((data = get(pDTitle)) == null || error != 0) {
			error += 60;
			throw new DIGAHttpException("リスト取得中、DIGAとのデータ送受信に失敗しました。",error);
		}
		
		Pattern p = Pattern.compile("(.*)location.href = \"(.+?)\";");
		Matcher m = p.matcher(data);
		if (! m.find()) { //リスト見つからない
			error = 65;
			throw new DIGAHttpDataFormatException("DIGAからの受信データが不正です。",error);
		}
		path = m.group(2);
		
		int pMax = -1, pCurrent = 0; 
		if ((data = get(path)) == null || error != 0) {
			error += 70;
			return null;
		}
		while (data.contains("DIGAと通信中です。しばらくお待ちください。")) {
			long stime = System.currentTimeMillis();
			if (lout != null) lout.appendLog("DIGAの起動を待っています。15秒後に再試行します。");
			while (System.currentTimeMillis() - stime < 15000) {
				//15秒待て
			}
			p = Pattern.compile("<input(.*?)name=\"(.+?)\"(.*?)value=\"(.*?)\"(.*?)>");
			m = p.matcher(data);
			buf = new StringBuffer(1000);
			while (m.find()) {
				buf.append(m.group(2) + "=" + m.group(4) + "&");
			}
			buf.deleteCharAt(buf.length()-1);
			if ((data = post("/cgi-bin/apl_err.cgi",buf.toString())) == null || error != 0) {
				throw new DIGAHttpException("リスト取得の再試行中、DIGAとのデータ送受信に失敗しました。",error);
			}
		}
		p = Pattern.compile("<option value=\"([0-9]+?)\">(.*?)</option>");
		m = p.matcher(data);
		while (m.find()) {
			try {
				pMax = Integer.parseInt(m.group(1));
			} catch(Exception e) {
				error = 66;
				throw new DIGAHttpDataFormatException("DIGAからの受信データが不正です。",error);
			}
		}
		p = Pattern.compile("><a(.|\\s)*?>([0-9]+?)</a(.|\\s)*?px\">(.*?)</font>(.|\\s)*?px\">(.*?)</font>(.|\\s)*?px\">(.*?)</font>(.|\\s)*?px\">(.*?)</font>(.|\\s)*?px\">(.*?)</font>(.|\\s)*?</tr>");
		m = p.matcher(data);
		while (m.find()) {
			result.append("\""+m.group(2)+"\",\""+m.group(4)+"\",\""+m.group(6)+"\",\""+((m.group(8)).replaceAll("&nbsp;"," "))+"\",\""+m.group(10)+"\",\""+((m.group(12)).replaceAll("\"","\"\""))+"\"\n");
		}
		
		if (pMax == -1) {
			error = 85;
			throw new DIGAHttpDataFormatException("DIGAからの受信データが不正です。",error);
		}
		
		String temp;
		pCurrent = 20;
		while (pCurrent <= pMax) {
			p = Pattern.compile("<input(.*?)name=\"(.+?)\"(.*?)value=\"(.*?)\"(.*?)>");
			m = p.matcher(data);
			buf = new StringBuffer(1000);
			while (m.find()) {
				temp = m.group(2);
				if (temp.equals("cCMD_VT_SELECT")) {
					buf.append(temp + "=" + pCurrent + "&");
				} else if (temp.startsWith("chk")) {
					buf.append(temp + "=&");
				} else {
					buf.append(temp + "=" + m.group(4) + "&");
				}
			}
			pdata = buf.substring(0, buf.length()-1);
			if (lout != null) lout.appendLog("リストを取得します 番号: " + pCurrent + " / " + pMax);
			
			if ((data = post("/cgi-bin/vttl_list.cgi", pdata)) == null || error != 0) {
				error += 85;
				throw new DIGAHttpException("リスト取得中、DIGAとのデータ送受信に失敗しました。",error);
			}
			
			p = Pattern.compile("><a(.|\\s)*?>([0-9]+?)</a(.|\\s)*?px\">(.*?)</font>(.|\\s)*?px\">(.*?)</font>(.|\\s)*?px\">(.*?)</font>(.|\\s)*?px\">(.*?)</font>(.|\\s)*?px\">(.*?)</font>(.|\\s)*?</tr>");
			m = p.matcher(data);
			while (m.find()) {
					result.append("\""+m.group(2)+"\",\""+m.group(4)+"\",\""+m.group(6)+"\",\""+((m.group(8)).replaceAll("&nbsp;"," "))+"\",\""+m.group(10)+"\",\""+((m.group(12)).replaceAll("\"","\"\""))+"\"\r\n");
			}
			pCurrent += 20;
		}
		
		error = 0;
		if (lout != null) lout.appendLog("リスト取得しました");
		return result.toString();
	}
}