// DigaListerConsole.java
// DIGA Lister (AWT) 1.1
// 2013.3.16 (SAT)
// 北の電話男 Kita no Denwa-otoko
// http://www.geocities.jp/kitano_denwaotoko/
//
// 参考
//  http://x68000.q-e-d.net/~68user/net/java-http-socket-1.html
//  http://www.javadrive.jp/regex/ini/index1.html
//  http://memories.zal.jp/WP/blog/20110228_1967.html, http://yan-note.blogspot.jp/2009/06/awt.html, http://wisdom.sakura.ne.jp/system/java/awt/Gjava32.html

import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class DigaLister extends Frame implements DIGAHttpLogDisplay, ActionListener, Runnable, FilenameFilter {
	private TextField hostField;
	private TextField passField;
	private Button runButton, saveButton;
	private TextArea logArea;
	private DIGAHttpCommunicator diga;
	private Thread getting;
	private String data;
	
	public void appendLog(String str) {
		logArea.append(str+"\n");
	}
	
	static public void main(String[] args)
	{
		new DigaLister("DIGA Lister 1.1");
	}
	
	public DigaLister(String title) {
		super(title);
		setSize(500,400);
		
		Frame cp = this;
		GridLayout layout = new GridLayout(0,2,10,10);
		Panel operation = new Panel();
		operation.setLayout(layout);
		cp.setLayout(new BorderLayout());
		
		Label hostLabel = new Label("DIGAのIPアドレス");
		Label passLabel = new Label("DIGAのパスワード");
		hostField = new TextField();
		passField = new TextField();
		runButton = new Button("リスト取得");
		saveButton = new Button("CSV保存");
		logArea = new TextArea();
		
		passField.setEchoChar('*');
		saveButton.addActionListener(this);
		saveButton.setActionCommand("SaveButton");
		runButton.addActionListener(this);
		runButton.setActionCommand("RunButton");
		saveButton.setEnabled(false);
		
		logArea.setEditable(false);
		logArea.setText("DIGA Lister 1.1\n\n北の電話男 http://www.geocities.jp/kitano_denwaotoko/\n\nDIGAの宅外リモート機能をONにして、\nDIGAのIPアドレスとパスワードを入力し、\n［リスト取得］ボタンを押して下さい。");
		
		operation.add(hostLabel);
		operation.add(hostField);
		operation.add(passLabel);
		operation.add(passField);
		operation.add(saveButton);
		operation.add(runButton);
		
		add(operation,BorderLayout.NORTH);
		add(logArea,BorderLayout.CENTER);
		
		try {
			BufferedReader br = new BufferedReader(new FileReader("./DLSettings"));
			String h = br.readLine();
			if (h != null) {
				hostField.setText(h);
			}
		} catch (IOException e) {
			//何もしない
		}
		
		diga = null;
		getting = null;

		//Runtime.getRuntime().addShutdownHook(new DIGAShutdownHook());//MacのJava VMでは終了メニューを使うと無条件に終了される。ログアウト処理を終了間際に
		//これをすると強制終了に問題が
		addWindowListener(new DIGAWindowAdapter());
		setVisible(true);
	}
	
	public void actionPerformed(ActionEvent e) {
		if ((e.getActionCommand()).equals("RunButton")) {
			logArea.setText("");
			runButton.setEnabled(false);
			saveButton.setEnabled(false);
			getting = new Thread(this);
			getting.start();
		} else if ((e.getActionCommand()).equals("SaveButton")) {
			saveCsv();
		}
	}
	
	public void run() {
		String host = hostField.getText();
		int port = 80;
		String password = passField.getText();
		BufferedWriter bw = null;
		
		try {
			diga = new DIGAHttpCommunicator(host,port,password);
			//diga.setPrintStream(System.out);
			diga.setLogDisplay(this);
			diga.setUserAgent("DIGALister/1.0 (HTTP List Getting)");
			
			if (! diga.login()) {
				int error = diga.getError();
				appendLog("*** DIGAへのログインに失敗しました エラー" + error);
				if (error == 26) appendLog("*** 認証に連続失敗した等の理由によりアクセスが禁止されています。\n*** しばらく待って再度操作して下さい。");
				else appendLog("*** パスワードが間違っている可能性があります");
				//System.exit(1);
				data = null;
				diga = null;
				appendLog("*** 処理終了しました");
				runButton.setEnabled(true);
				if (data == null) saveButton.setEnabled(false);
				else saveButton.setEnabled(true);
				return;
			}
		
			try { //正常ログインできたらそのアドレスを記録
				BufferedWriter sw = new BufferedWriter(new FileWriter("./DLSettings"));
				sw.write(host + '\n');
				sw.close();
			} catch (IOException e) {
				System.err.print("設定ファイル書込み失敗\n");
			}
			
			appendLog("*** リストの取得を開始します");
			if ((data = diga.getDTitleCSV()) == null) {
				appendLog("*** リストの取得に失敗しました エラー" + diga.getError() + "");
			} else {
				appendLog("*** リストを取得しました");
				logArea.append(data);
			}
			
			DIGAHttpCommunicator diga = this.diga;
			this.diga = null;
			appendLog("*** DIGAからログアウトします");
			if (! diga.logout()) {
				appendLog("*** DIGAからのログアウトに失敗しました エラー" + diga.getError() + "");
				//System.exit(1);
			} else {
				appendLog("*** DIGAからログアウトしました");
			}
		} catch (DIGAHttpException e) {
			logArea.append(e.getMessage()+"\n");
		}
		diga = null;
		appendLog("*** 処理終了しました");
		
		runButton.setEnabled(true);
		if (data == null) saveButton.setEnabled(false);
		else saveButton.setEnabled(true);
	}
	
	void saveCsv() {
		if (data == null) return;
		FileDialog dialog = new FileDialog(this,"CSVに保存 .csvで終るファイル名にして下さい",FileDialog.SAVE);
		dialog.setFilenameFilter(this);
		dialog.setFile(".csv");
		dialog.setVisible(true);
		String fn = dialog.getDirectory();
		if (fn != null) {
			fn += dialog.getFile();
			try {
				BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(fn), "Shift_JIS"));
				bw.write(data);
				bw.close();
				appendLog("ファイルに保存しました " + fn);
			} catch (IOException e) {
				appendLog("ファイル書込み失敗しました " + fn);
			}
		}
	}
	
	public boolean accept(File dir, String name) { //あまり意味はなさそうだが、ファイルタイプフィルタ
		return name.endsWith(".csv");
	}
	
	private class DIGAShutdownHook extends Thread { //MacのJava VMでは終了メニューで無条件に終了される。ログアウト処理を終了間際に
		public DIGAShutdownHook() {
			super();
		}
		public void run() { 
			if (diga != null) {
				try {
					diga.logout();
				} catch(DIGAHttpException e) {
					//もう何もしない
				}
				diga = null;
			}
		}
	}
	
	private class DIGAWindowAdapter extends WindowAdapter {
		public void windowClosing(WindowEvent we) {
			if (diga != null) {
				try {
					diga.logout();
				} catch(DIGAHttpException e) {
					//もう何もしない
				}
				diga = null;
			}
			System.exit(0);
		}
	}
}